"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.babelImportToRequire = void 0;
const resolve_1 = __importDefault(require("resolve"));
function viteReactJsx() {
    return {
        name: 'vite:react-jsx',
        enforce: 'pre',
        config: () => ({
            resolve: {
                dedupe: ['react', 'react-dom'],
            },
        }),
        configResolved(config) {
            if (config.command === 'build') {
                Object.assign(this, getRuntimeLoader(config));
                this.transform = getTransformer({
                    sourceMaps: !!config.build.sourcemap,
                });
            }
            else {
                const jsxRE = /\.[tj]sx$/;
                const reactRE = /(^|\n)import React[ ,]/;
                // Just use React.createElement in serve mode
                this.transform = function (code, id) {
                    if (jsxRE.test(id) && !reactRE.test(code)) {
                        return `import React from 'react'; ` + code;
                    }
                };
            }
        },
    };
}
exports.default = viteReactJsx;
viteReactJsx.getRuntimeLoader = getRuntimeLoader;
viteReactJsx.restoreJSX = restoreJSX;
function getRuntimeLoader(opts) {
    const runtimeId = 'react/jsx-runtime';
    return {
        name: 'vite:react-jsx',
        enforce: 'pre',
        resolveId(id) {
            return id === runtimeId ? id : null;
        },
        load(id) {
            if (id === runtimeId) {
                const runtimePath = resolve_1.default.sync(runtimeId, {
                    basedir: opts.root,
                });
                const exports = ['jsx', 'jsxs', 'Fragment'];
                return [
                    `import * as jsxRuntime from '${runtimePath}'`,
                    // We can't use `export * from` or else any callsite that uses
                    // this module will be compiled to `jsxRuntime.exports.jsx`
                    // instead of the more concise `jsx` alias.
                    ...exports.map(name => `export const ${name} = jsxRuntime.${name}`),
                ].join('\n');
            }
        },
    };
}
function getTransformer(opts) {
    const babelImport = Promise.resolve().then(() => __importStar(require('@babel/core')));
    const babelTransformJsx = Promise.resolve().then(() => __importStar(require('@babel/plugin-transform-react-jsx')));
    return async function transform(code, id) {
        if (/.+\/node_modules\/.+\.jsx?$/.test(id)) {
            const babel = await babelImport;
            // Reverse-compile any React.createElement calls
            let [ast, isCommonJS] = await viteReactJsx.restoreJSX(babel, code);
            // Then apply the JSX automatic runtime transform
            if (ast) {
                const plugins = [
                    [await babelTransformJsx, { runtime: 'automatic' }],
                ];
                if (isCommonJS) {
                    plugins.push(babelImportToRequire);
                }
                const result = await babel.transformFromAstAsync(ast, undefined, {
                    plugins,
                    sourceMaps: opts.sourceMaps,
                });
                if (result === null || result === void 0 ? void 0 : result.code) {
                    return {
                        code: result.code,
                        map: result.map,
                    };
                }
            }
        }
        else if (/\.[tj]sx$/.test(id)) {
            const syntaxPlugins = [];
            if (id.endsWith('.tsx')) {
                syntaxPlugins.push(await babelTSX());
            }
            const babel = await babelImport;
            const res = await babel.transformAsync(code, {
                plugins: [
                    ...syntaxPlugins,
                    [await babelTransformJsx, { runtime: 'automatic' }],
                ],
                sourceMaps: opts.sourceMaps,
            });
            if (res === null || res === void 0 ? void 0 : res.code) {
                return {
                    code: res.code,
                    map: res.map,
                };
            }
        }
    };
    async function babelTSX() {
        return [
            await Promise.resolve().then(() => __importStar(require('@babel/plugin-syntax-typescript'))).then(m => m.default),
            { isTSX: true },
        ];
    }
}
let babelRestoreJSX;
/** Restore JSX from `React.createElement` calls */
async function restoreJSX(babel, code) {
    const [reactAlias, isCommonJS] = parseReactAlias(code);
    const reactJsxRE = new RegExp('\\b' + reactAlias + '\\.(createElement|Fragment)\\b', 'g');
    let hasCompiledJsx = false;
    code = code.replace(reactJsxRE, (_, prop) => {
        hasCompiledJsx = true;
        // Replace with "React" so JSX can be reverse compiled.
        return 'React.' + prop;
    });
    if (!hasCompiledJsx) {
        return [null, false];
    }
    // Support modules that use `import {Fragment} from 'react'`
    code = code.replace(/createElement\(Fragment,/g, 'createElement(React.Fragment,');
    babelRestoreJSX || (babelRestoreJSX = Promise.resolve().then(() => __importStar(require('./babelRestoreJsx'))));
    const result = await babel.transformAsync(code, {
        ast: true,
        code: false,
        parserOpts: {
            plugins: ['jsx'],
        },
        plugins: [await babelRestoreJSX],
    });
    return [result === null || result === void 0 ? void 0 : result.ast, isCommonJS];
}
function parseReactAlias(code) {
    let match = code.match(/\b(var|let|const) +(\w+) *= *require\(["']react["']\)/);
    if (match) {
        return [match[2], true];
    }
    match = code.match(/^import (\w+).+? from ["']react["']/m);
    if (match) {
        return [match[1], false];
    }
    return [undefined, false];
}
/**
 * Replace this:
 *
 *     import { jsx as _jsx } from "react/jsx-runtime"
 *
 * with this:
 *
 *     var _jsx = require("react/jsx-runtime").jsx
 */
function babelImportToRequire({ types: t, }) {
    return {
        visitor: {
            ImportDeclaration(path) {
                const decl = path.node;
                const spec = decl.specifiers[0];
                path.replaceWith(t.variableDeclaration('var', [
                    t.variableDeclarator(spec.local, t.memberExpression(t.callExpression(t.identifier('require'), [decl.source]), spec.imported)),
                ]));
            },
        },
    };
}
exports.babelImportToRequire = babelImportToRequire;
//# sourceMappingURL=plugin.js.map